"""
Code taken and adapted from the article :https://towardsdatascience.com/word2vec-for-phrases-learning-embeddings-for-more-than-one-word-727b6cf723cf

This file builds the phrases model for bigrams. These can be used to train our own Word2Vec model with unigrams and
bigrams. See train_w2v_vocabuylary.py
"""

import json
import nltk
import os
import re
from gensim.models import Word2Vec
from gensim.models.phrases import Phrases, Phraser
from spacy.lang.en.stop_words import STOP_WORDS

from es_service import insert_vocab

# nltk.download('punkt')
lemmatizer = nltk.stem.WordNetLemmatizer()


def remove_punct(tokens):
    no_punct = []
    for token in tokens:
        token = re.sub(r'[^a-z]', '', token)
        if token and len(token) > 1:
            no_punct.append(token)

    return no_punct


def build_phrases(sentences):
    phrases = Phrases(sentences,
                      min_count=5,
                      threshold=3,
                      progress_per=1000)
    return Phraser(phrases)


def get_sentences(text):
    sentences = nltk.tokenize.sent_tokenize(text.lower())
    lemmatized_sentences = []
    for sentence in sentences:
        tokens = sentence.split()
        cleaned_tokens = remove_punct(tokens)
        lemmatized_tokens = [lemmatizer.lemmatize(token) for token in cleaned_tokens if token not in STOP_WORDS]
        # don't add empty lists
        if lemmatized_tokens:
            lemmatized_sentences.append(lemmatized_tokens)
    return lemmatized_sentences


def sentence_to_bi_grams(phrases_model, sentence):
    return ' '.join(phrases_model[sentence])


def loop_through_files(folders):
    lemmatized_sentences = []
    for folder in os.listdir(folders):
        path = folders + '/' + folder + '/'
        for file in os.listdir(path):
            with open(path + file, 'r', encoding="utf8") as f:
                data = json.load(f)
                # some docs have response as null
                if data['response']:
                    # replace blank lines with dots as titles usually don't have period to indicate the end of sentence
                    lemmas = get_sentences(data['response'].replace('\n', '. '))
                    # discard empty lists
                    if lemmas:
                        lemmatized_sentences += lemmas
    return lemmatized_sentences


# create phrases
lemmatized_sentences = loop_through_files('./out')
phrases = build_phrases(lemmatized_sentences)
# phrases.save('phrases_model.txt')

# build w2v vocab
lemmatized_corpus = phrases[lemmatized_sentences]
w2v_model = Word2Vec(lemmatized_corpus, size=300, window=5, min_count=2)
w2v_model.save('SearchEngine/w2v_vocab.model')
# To load a saved model for efficient use
# w2v_model = KeyedVectors.load('w2v_vocab.model')
# To load a saved model for further training
# w2v_model = Word2Vec.load('w2v_vocab.model')

# insert vocabulary into ES
# normalise vectors
w2v_model.init_sims(replace=True)

print('Inserting vocabulary')
insert_vocab(w2v_model)
