import itertools
import json
import numpy as np
import os
import pandas as pd
import re
import spacy
from fuzzywuzzy import fuzz
from scrapy.exceptions import DropItem

from databot.config import *

# python -m spacy download en_core_web_sm
# python -m spacy init-model en /tmp/la_vectors_wiki_lg --vectors-loc cc.la.300.vec.gz
# https://s3-us-west-1.amazonaws.com/fasttext-vectors/wiki-news-300d-1M-subword.bin.zip

path = './out/AneurinBevan/'
labels = ['board meeting',
          'audit committee',
          'charitable funds committee',
          'digital and information management committee',
          'quality and safety committee',
          'finance and performance committee',
          'remuneration committee',
          'local partnership forum',
          'mental health act committee',
          'stakeholder reference group',
          'health professionals forum',
          'strategy committee',
          'health and safety committee',
          'workforce committee']

nlp = spacy.load('tmp/')

class CommitteeTypePipeline(object):

    def process_item(self, item, spider):

        if spider.name == 'AneurinBevan':
            # newer resources are split by item type tables whilst old resources are public board meetings
            label = {item['helper_committee_type']} if item['helper_committee_type'] else {'public board meetings'}
        elif spider.name in ['BetsiCadwaladr', 'Cavuhb']:
            label = {item['helper_committee_type']}
        elif spider.name in ['CardiffVale', 'CwmTafMorgannwg', 'HywelDda', 'Powys', 'SwanseaBay']:
            label = set(item['helper_committee_type'])
        # label = {data['referer_html']} if data['referer_html'] else {'public board meetings'}
        label = {' '.join(re.findall(r'[a-z]+', i.lower())) for i in label}
        committee_type = None
        if label.intersection(OTHER):
            committee_type = 'other'
        else:
            # sometimes some strings have no letters, make tuples only from non empty items
            data = [list(itertools.product([i], labels)) for i in label if i]
            # 'SpaCy approach with FastText'
            df = pd.DataFrame(data, columns=labels)
            df = df.applymap(lambda x: nlp(x[0]).similarity(nlp(x[1])))
            # if committee type is less than 65%, then drop the item as it should be a committee type
            # that we are not interested to keep
            score = df.max().max()
            if score < 0.65:
                raise DropItem('Committee type score too low {}'.format(score))
            df['max'] = df.max().idxmax(axis=1)
            committee_type = df['max'].values[0]
            print('Committee type: {}, {}'.format(committee_type, score))

        item['committee_type'] = committee_type

        return item
