import re
from fuzzywuzzy import fuzz
from lxml import etree
from scrapy.exceptions import DropItem

from databot.config import *


class CommitteeTypePipeline(object):

    def process_item(self, item, spider):
        """
        The committee types are hard coded in the config file and mapped in this class. This way we can ensure
        the accuracy of the item types. However, this programmatically is not a nice solution. Could try some fuzzy
        string matching instead. It is difficult to write something generic when each website use completely different
        committee types and different markup

        For some spiders the committee type can be reduced to single string search but for some,
        the type might appear in a list of items (eg Hywel Dda). To speed up searching lists, use sets instead
        """

        label = ''
        committee_type = None
        if spider.name == 'AneurinBevan':
            # newer resources are split by item type tables whilst old resources are public board meetings
            label = {item['referer_html']} if item['referer_html'] else {'public board meetings'}
        elif spider.name == 'BetsiCadwaladr':
            label = {item['referer_html']}
        elif spider.name == 'CardiffVale':
            label = {item['referer_url'].split('/')[-1]}
        elif spider.name == 'CwmTafMorgannwg':
            label = {item['referer_url'].split('/')[5]}
        elif spider.name in ['HywelDda', 'Powys', 'SwanseaBay']:
            label = {item['helper_committee_type']}
        print(label)

        label = {' '.join(re.findall(r'[a-z]+', i.lower())) for i in label}
        print(label)
        if label.intersection(BOARD_MEETINGS):
            committee_type = 'board meeting'
        elif label.intersection(AUDIT_COMMITTEE):
            committee_type = 'audit committee'
        elif label.intersection(CHARITABLE_FUNDS_COMMITTEE):
            committee_type = 'charitable funds committee'
        elif label.intersection(DIGITAL_INFORMATION_MANAGEMENT_COMMITTEE):
            committee_type = 'digital and information management committee'
        elif label.intersection(QUALITY_SAFETY_COMMITTEE):
            committee_type = 'quality and safety committee'
        elif label.intersection(FINANCE_PERFORMANCE_COMMITTEE):
            committee_type = 'finance and performance committee'
        elif label.intersection(REMUNERATION_COMMITTEE):
            committee_type = 'remuneration committee'
        elif label.intersection(LOCAL_PARTNERSHIP_FORUM):
            committee_type = 'local partnership forum'
        elif label.intersection(MENTAL_HEALTH_ACT_COMMITTEE):
            committee_type = 'mental health act committee'
        elif label.intersection(STAKEHOLDER_REFERENCE_GROUP):
            committee_type = 'stakeholder reference group'
        elif label.intersection(HEALTH_PROFESSIONALS_FORUM):
            committee_type = 'health professionals forum'
        elif label.intersection(STRATEGY_COMMITTEE):
            committee_type = 'strategy committees'
        elif label.intersection(HEALTH_SAFETY_COMMITTEES):
            committee_type = 'health and safety committee'
        elif label.intersection(WORKFORCE_COMMITTEE):
            committee_type = 'workforce committee'
        elif label.intersection(OTHER):
            committee_type = 'other'
        else:
            DropItem()
        print('[] [] [] [] [] ')
        print(committee_type)


        item['committee_type'] = committee_type

        return item
