import dateparser
import re
from datetime import date
from scrapy.exceptions import DropItem
from urllib.parse import urlparse, unquote

# to avoid matching 3 04.04 in string 'QPSC Board Book V3 04.04.19.pdf'
# we have to ensure that the same separator is used consistently
# considers formats dd_mm_yy, dd_mm_yyyy, yy_mm_dd, yyyy_mm_dd, mm_dd_yyyy
r = '((0?[1-9]|0[1-9]|1[0-9]|2[0-9]|3[0-1])(st|nd|rd|th)?'\
    '{0}(0?[1-9]|1[0-2]|(January|February|March|April|May|June|July|August|September|October|November|December)?|(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec)?)'\
    '{0}((20)?([0-1]\d|2[0-1])))'\
    '|(((20)?([0-1]\d|2[0-1]))'\
    '{0}(0?[1-9]|1[0-2])'\
    '{0}(0[1-9]|1[0-9]|2[0-9]|3[0-1]))'\
    '|((January|February|March|April|May|June|July|August|September|October|November|December)'\
    '{0}(0?[1-9]|1[0-9]|2[0-9]|3[0-1])'\
    '{0}((20)?([0-1]\d|2[0-1])))'

r_updated = ''
separators = ['\.', '-', '\s', '/', ' ']

for separator in separators:
    r_updated += r.format(separator)
    if separator != separators[-1]:
        r_updated += '|'

class DatePipeline(object):

    def process_item(self, item, spider):
        # drop html items
        if 'text/html' in item['content_type']:
            raise DropItem('Drop HTML item')

        item['date'] = None
        path = unquote(urlparse(item['url']).path)
        # as urls are of format
        # '/Docs/Board_Papers/Legacy 2015-2016/16-03 March 2016/AI 5.1  Appendix 2 Chairs report FP&W for UHB 2 March 16.pdf'
        # and regex would match '03 March 2016' first
        if spider.name == 'CwmTafMorgannwg':
            path = path.split('/')[-1]
        elif spider.name in ['AneurinBevan', 'CardiffVale']:
            path = item['helper_date']
        date = re.search(r_updated, path, flags=re.IGNORECASE)

        # for powys spider, if there is no date found in url,
        # sometimes the meeting date can be found in the referer's url
        if not date and spider.name in ['BetsiCadwaladr', 'CwmTafMorgannwg', 'HywelDda', 'Powys', 'SwanseaBay']:
            date = re.search(r_updated, item['helper_date'], flags=re.IGNORECASE)
        if date:
            item['date'] = dateparser.parse(date.group(), date_formats=['%d.%m.%Y', '%d-%m-%Y', '%Y-%m-%d'],
                                                          settings={'DATE_ORDER': 'DMY'}).strftime('%Y/%m/%d')
            print('Date: ' + str(item['date']))

        return item
