import json
import numpy as np
import os
import pandas as pd
import re
import spacy
from fuzzywuzzy import fuzz
from nltk import bigrams
import itertools



labels = ['draft minutes', 'final minutes', 'agenda', 'papers']
# load fastText vectors
nlp = spacy.load('tmp/')

# measure similarity between grams and bigrams instead of n-grams and full sentences

class ItemTypePipeline(object):

    def process_item(self, item, spider):

        print('Helper Item Type: ' + item['helper_item_type'])
        unigrams = re.findall(r'[a-z]+', item['helper_item_type'].lower())
        # in betsi spider for agendas there is no /text() label. In that case look in the last url path
        if not unigrams:
            item['helper_item_type'] = list(filter(None, item['url'].split('/')))[-1]
            return self.process_item(item, spider)

        bigrams_ = [' '.join(bigram) for bigram in bigrams(unigrams)]
        grams = unigrams + bigrams_

        data = [list(itertools.product([i], labels)) for i in grams]
        df = pd.DataFrame(data, columns=labels)
        df = df.applymap(lambda x: nlp(x[0]).similarity(nlp(x[1])))
        score = df.max().max()

        if score < 0.5 and spider.name == 'BetsiCadwaladr':
            item['helper_item_type'] = list(filter(None, item['url'].split('/')))[-1]
            return self.process_item(item, spider)

        # if the confidence is smaller than 65%, assign the generic label of 'papers'
        # the chosen number based on xsl spreadsheet
        if df.max().max() < 0.65:
            item['item_type'] = 'papers'
        else:
            df['max'] = df.max().idxmax(axis=1)

            item['item_type'] = df['max'].values[0]

        print('Item Type: {}, {}'.format(item['item_type'], score))
        return item
