import hashlib
import io
import json
import os
import zipfile
from lxml import etree
from tika import parser
from urllib.parse import urlparse
from xml.etree.ElementTree import XML

WORD_NAMESPACE = '{http://schemas.openxmlformats.org/wordprocessingml/2006/main}'
PARA = WORD_NAMESPACE + 'p'
TEXT = WORD_NAMESPACE + 't'


class JsonWriterPipeline:

    def process_item(self, item, spider):

        # create directory named like spider if it does not exist
        directory = 'out/' + spider.name
        if not os.path.exists(directory):
            os.makedirs(directory)

        if 'text/html' not in item['content_type']:

            if 'application/vnd.openxmlformats-officedocument' in item['content_type']:
                print('Writing Word Doc')
                # Get the document XML from the zip container
                document = zipfile.ZipFile(io.BytesIO(item['response']), "r")
                xml_content = document.read('word/document.xml')
                document.close()
                tree = XML(xml_content)

                # Parse each paragraph in the document
                paragraphs = []
                for paragraph in tree.iter(PARA):
                    texts = [node.text
                             for node in paragraph.iter(TEXT)
                             if node.text]
                    if texts:
                        paragraphs.append(''.join(texts))

                item['response'] = '\n\n'.join(paragraphs)

            if 'application/pdf' in item['content_type']:
                print('Writing PDF')
                pdf = parser.from_buffer(item['response']).get('content')
                item['response'] = pdf

            h = hashlib.md5(item['url'].encode()).hexdigest()
            path = directory + '/{}.json'.format(h)
            with open(path, "w") as f:
                f.write(json.dumps(dict(item)))

        return item['url']
