import lxml.html
import scrapy
from scrapy.http import Request

from scrapy.linkextractors import LinkExtractor

from databot.item import Item


class AneurinBevanSpider(scrapy.Spider):
    name = 'AneurinBevan'
    allowed_domains = ['wales.nhs.uk']
    start_urls = ['http://www.wales.nhs.uk/sitesplus/866/page/41395']

    def parse(self, response):
        # committee category mentioned in the referer page
        # create manual requests to preserve them
        # newer papers have been organised in tables whilst older ones is a large list of public board meetings
        # the starting url has board meetings from recent years and has links to older papers
        table_headers = response.xpath('//div[@id="pagecontent"]//h3/text()').getall()
        tables = response.xpath('//div[@id="pagecontent"]//table').getall()
        if table_headers:
            for i in range(len(tables)):
                # select rows where a link appears
                rows = lxml.html.fromstring(tables[i]).xpath('//tr[descendant::a]')
                for row in rows:
                    # for whatever reason row.xpath('/td[1]//text()') does not work
                    # replace td with child::node() instead
                    date = ' '.join(row.xpath('child::node()[1]//text()'))
                    for url in row.xpath('child::node()[4]//a'):
                        yield Request(
                            url=response.urljoin(url.xpath('@href')[0]),
                            callback=self.parse_item,
                            cb_kwargs=dict(helper_committee_type=table_headers[i],
                                           helper_item_type=url.xpath('text()')[0],
                                           helper_date=date)
                        )
        else:
            text = response.xpath('(//div[@id="pagecontent"]|//table[@class="inputform"])//a[contains(@href,"opendoc")]//text()').getall()
            urls = response.xpath('(//div[@id="pagecontent"]|//table[@class="inputform"])//a[contains(@href,"opendoc")]/@href').getall()
            date = response.xpath('//select[@id="libraryid"]/option[@selected="selected"]/text()').get()
            for i in range(len(urls)):
                yield Request(
                    url=response.urljoin(urls[i]),
                    callback=self.parse_item,
                    cb_kwargs=dict(helper_item_type=text[i],
                                   helper_date=date)
                )

            # for old archived documents
            for href in response.xpath('//div[@id="pagecontent"]//li//a[contains(@href,"documentmap")]/@href').getall():
                # print(href)
                yield Request(
                    url=response.urljoin(href),
                    callback=self.parse
                )

        # for Agendas and Papers 2018 and Agendas and Papers prior to 2018
        for href in response.xpath('//div[@id="pagecontent"]//li//a[contains(@href,"page")]/@href').getall():
            yield Request(
                url=response.urljoin(href),
                callback=self.parse
            )

    def parse_item(self, response, helper_item_type, helper_date, helper_committee_type=None):
        item = Item()
        item['name'] = self.name
        item['url'] = response.url
        item['response'] = response.body
        item['helper_committee_type'] = helper_committee_type
        item['helper_item_type'] = str(helper_item_type)
        item['helper_date'] = helper_date
        item['content_type'] = response.headers['Content-Type'].decode()
        item['last_modified'] = response.headers['Last-Modified'].decode()
        yield item
