import scrapy
from more_itertools import unique_everseen
from scrapy.http import Request

from databot.item import Item


class BetsiCadwaladrSpider(scrapy.Spider):
    name = 'BetsiCadwaladr'
    allowed_domains = ['bcuhb.nhs.wales']

    def start_requests(self):
        # for board meetings
        yield Request('https://bcuhb.nhs.wales/about-us/health-board-meetings-and-members/health-board-meetings/',
                      self.parse_html, cb_kwargs=dict(helper_committee_type='board meeting'))
        # for committees
        yield Request('https://bcuhb.nhs.wales/about-us/committees-and-advisory-groups/', self.parse)

    def parse(self, response):
        committee_types = response.xpath('//table//strong/text()').getall()
        # some cells has very poor html as some invisible url duplicates appear
        # use library as converting to sets will loose order of elements
        urls = list(unique_everseen(response.xpath('//table//a/@href').getall()))
        for i in range(len(urls)):
            yield Request(
                url=response.urljoin(urls[i]),
                callback=self.parse_html,
                cb_kwargs=dict(helper_committee_type=committee_types[i])
            )

    def parse_html(self, response, helper_committee_type):
        # create manual requests to preserve committee type
        # item type sometimes mentioned in the <a> tag. If not, the item is an agenda
        helper_item_type = response.xpath('//table//a//text()').getall()
        urls = response.xpath('//table//a/@href').getall()
        for i in range(len(urls)):
            yield Request(
                url=response.urljoin(urls[i]),
                callback=self.parse_item,
                cb_kwargs=dict(helper_item_type=helper_item_type[i],
                               helper_committee_type=helper_committee_type)
            )

    def parse_item(self, response, helper_item_type, helper_committee_type):
        item = Item()
        item['name'] = self.name
        item['url'] = response.url
        item['response'] = response.body
        item['content_type'] = response.headers['Content-Type'].decode()
        item['last_modified'] = response.headers['Last-Modified'].decode()
        item['helper_item_type'] = helper_item_type
        item['helper_committee_type'] = helper_committee_type
        yield item
