from scrapy.http import Request
from scrapy.linkextractors import LinkExtractor
from scrapy.spiders import CrawlSpider, Rule

from databot.item import Item


class CardiffValeSpider(CrawlSpider):

    # everything before 2015 has a different, even more nested structure
    # the second link will follow to the board meetings
    name = 'CardiffVale'
    allowed_domains = ['cardiffandvaleuhb.wales.nhs.uk']
    start_urls = ['http://www.cardiffandvaleuhb.wales.nhs.uk/board-committees-and-advisory-groups']

    rules = (
        Rule(LinkExtractor(deny=('/opendoc'),
                           restrict_xpaths=(['//div[@id="pagecontent"]//ul',
                                             '//div[@id="pagecontent"]//a[contains(@href,"papers")]',
                                             '//div[@id="pagecontent"]//a[contains(@href,"minutes")]'
                                             ])), callback='parse_html', follow=True),
        Rule(LinkExtractor(allow=('/board-meetings')), callback='parse_html', follow=False),
    )

    def parse_html(self, response, before_2015=False):

        # sometimes dates are not in the <a> tag but in <a><u><font>: add // before text()
        # the urls are usually in tables, except for meeting minutes when its just a list
        urls = response.xpath('//div[@id="pagecontent"]//table//a[contains(@href,"opendoc")]/@href').getall()
        dates = response.xpath('//div[@id="pagecontent"]//table//a[contains(@href,"opendoc")]//text()').getall()
        if not urls:
            urls = response.xpath('//div[@id="pagecontent"]//ul//a/@href').getall()
            dates = response.xpath('//div[@id="pagecontent"]//ul//a//text()').getall()

        # in the main page there is no item type mentioned, the documents are joined agendas with papers
        helper_item_type = 'papers'
        if any(string in response.url for string in ['papers', 'minutes']):
            helper_item_type = response.xpath('//h1/text()').get()

        if before_2015:
            print('looped 2015')
            date = response.xpath('//h1/text()').get()
            for i in range(len(urls)):
                yield Request(
                    url=response.urljoin(urls[i]),
                    callback=self.parse_item,
                    cb_kwargs=dict(helper_date=date,
                                   helper_item_type=helper_item_type,
                                   referer_html=response.text)
                )

        else:
            for i in range(len(urls)):
                yield Request(
                    url=response.urljoin(urls[i]),
                    callback=self.parse_item,
                    cb_kwargs=dict(helper_date=dates[i],
                                   helper_item_type=helper_item_type,
                                   referer_html=response.text)
                )

        # archived papers from before 2015 has a different structure
        # audit committee use base_url/page/number
        # other committees use base_url/committee-name-date
        for url in response.xpath('//div[@id="pagecontent"]//table//a[not(contains(@href,"opendoc"))]/@href').getall():
            yield Request(
                url=response.urljoin(url),
                callback=self.parse_html,
                cb_kwargs=dict(before_2015=True)
            )

    def parse_item(self, response, helper_item_type, helper_date, referer_html):
        item = Item()
        item['name'] = self.name
        item['url'] = response.url
        item['response'] = response.body
        item['referer_url'] = response.request.headers.get('Referer').decode()
        item['referer_html'] = referer_html
        item['helper_date'] = helper_date
        item['helper_item_type'] = helper_item_type
        item['content_type'] = response.headers['Content-Type'].decode()
        if response.headers.get('Last-Modified'):
            item['last_modified'] = response.headers.get('Last-Modified').decode()
        yield item
