from scrapy.http import Request
from scrapy.linkextractors import LinkExtractor
from scrapy.spiders import CrawlSpider, Rule

from databot.item import Item


class CwmTafMorgannwgSpider(CrawlSpider):
    name = 'CwmTafMorgannwg'
    allowed_domains = ['cwmtafmorgannwg.wales', 'cwmtafuhb.wpengine.com']
    start_urls = ['https://cwmtafmorgannwg.wales/how-we-work/decision-making-2/']

    rules = (
        Rule(LinkExtractor(restrict_xpaths=(['//div[@id="main"]//ul'])), callback='parse_pdf',
             follow=False),
    )

    # create own requests directly to pdfs as scrapy does not follow links where download is enabled
    def parse_pdf(self, response):
        urls = response.xpath('//table/tbody/tr/td[2]//a/@href').getall()
        # only the second column has a usable text() tag. The url in first and second column are identical
        helper_item_types = response.xpath('//table/tbody/tr/td[2]//a//text()').getall()
        for i in range(len(urls)):
            if urls[i].endswith('.pdf'):
                # sometimes date mentioned in the breadcrumbs but not always
                date = ' '.join(response.xpath('//div[contains(@class, "crumbs")]//text()').getall())
                yield Request(
                    url=urls[i],
                    callback=self.parse_item,
                    cb_kwargs = dict(helper_item_type=helper_item_types[i],
                                     helper_date=date)
                )
            # as we don't know how many levels of directories there are, reyield the non pdf requests
            else:
                yield Request(
                    url=urls[i],
                    callback=self.parse_pdf
                )

    def parse_item(self, response, helper_item_type, helper_date):
        item = Item()
        item['name'] = self.name
        item['url'] = response.url
        item['response'] = response.body
        item['referer_url'] = response.request.headers.get('Referer').decode()
        item['helper_item_type'] = helper_item_type
        item['helper_date'] = helper_date
        item['content_type'] = response.headers['Content-Type'].decode()
        item['last_modified'] = response.headers.get('Last-Modified').decode()
        yield item
