from scrapy.http import Request
from scrapy.linkextractors import LinkExtractor
from scrapy.spiders import CrawlSpider, Rule

from databot.item import Item


class HywelDdaSpider(CrawlSpider):
    name = 'HywelDda'
    allowed_domains = ['wales.nhs.uk']
    # one url for board meetings and the other for committees
    start_urls = ['http://www.wales.nhs.uk/sitesplus/862/page/40875',
                  'http://www.wales.nhs.uk/sitesplus/862/page/72048']

    rules = (
        Rule(LinkExtractor(allow=('862/page',), restrict_xpaths=(['//div[@id="pagecontent"]'])),
                           callback='parse_html', follow=True),
    )

    def parse_html(self, response):
        # to preserve the date mentioned in the html <h1> tag and committee type,
        # create manual requests to the 'opendoc' urls
        # and add an extra attribute with the referer's html
        # some icons also contains urls with no text to the same documents as the link followed the icon
        urls = response.xpath('//div[@id="content"]//a[parent::span|parent::td and not(contains(@href, "document"))]/@href').getall()
        helper_item_type = response.xpath('//div[@id="content"]//a[parent::span|parent::td]//text()').getall()
        for i in range(len(urls)):
            if 'opendoc/' in urls[i]:
                yield Request(
                    url=response.urljoin(urls[i]),
                    callback=self.parse_item,
                    cb_kwargs=dict(referer_html=response.text,
                                   helper_item_type=helper_item_type[i])
                )

    def parse_item(self, response, referer_html, helper_item_type):
        item = Item()
        item['name'] = self.name
        item['url'] = response.url
        item['response'] = response.body
        item['referer_html'] = referer_html
        item['helper_item_type'] = helper_item_type
        item['content_type'] = response.headers['Content-Type'].decode()
        if response.headers.get('Last-Modified'):
            item['last_modified'] = response.headers.get('Last-Modified').decode()
        yield item
